// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2003 - 2006 Simon Goodall

package WFUT;

import java.io.*;
import java.net.*;
import java.util.*;

import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

/**
 * This class handles most of the file IO for WFUT
 */
class FileIO {
  /**
   * This method returns a list of channel objects.
   * This reads channel information from the file specified
   * in a URL (src) and returns a list of Channel objects.
   * @see Channel
   * @param src URL of channel list
   * @return List of Channel objects
   */
  public static List getChannels(final String src) throws Exception {
    final URL url = new URL(src);
    final ChannelParser p = new ChannelParser();
    p.parse(url.openStream());
    return p.getChannels();
  }

  /**
   * This method returns a list of FileObjects for a channel.
   * This reads a channel file listing from the URL specified in 
   * channel and returns a map object of FileObjects keyed by the filename.
   * @see FileObject
   * @param channel URL for channel file list
   * @return Map of FileObjects keyed by filename
   */
  public static ChannelList getChannelList(final String channel) throws MalformedURLException, ParserConfigurationException, SAXException, IOException {
    final URL url = new URL(channel);
    final FileParser p = new FileParser();
    p.parse(url.openStream());
    return p.getFileMap();
  }

  /**
   * This method returns a list of FileObjects for a channel from local 
   * install.
   * This reads a channel file listing from the filename specified in 
   * channel and returns a map object of FileObjects keyed by the filename.
   * @see FileObject
   * @param channel URL for channel file list
   * @return Map of FileObjects keyed by filename
   */
  public static ChannelList getLocalList(final String filename) throws FileNotFoundException, IOException, ParserConfigurationException, SAXException {
    final FileParser p = new FileParser();
    p.parse(new FileInputStream(new File(filename)));
    return p.getFileMap();
  }

  /**
   * This method saves the local channel file list.
   * This saves a local channel file listing from a list of FileObjects
   * @see FileObject
   * @param newList List of FileObjects
   * @param filename Filename to save to
   */
  public static void saveLocalList(final ChannelList newList, final String filename) throws IOException {
    // Create directory if required
    if (filename.lastIndexOf('/') > 0) {
      final File f = new File(filename.substring(0, filename.lastIndexOf('/')));
      if (!f.exists()) {
        f.mkdirs();
      }
    }

    FileObjectWriter.writeFileList(filename, newList);
  }

  /**
   * This method saves a FileObject to the temp channel file.
   * This method is used to save downloaded files to a tempory channel listing
   * file in case the local listing file does not get saved for some reason.
   * @param channel Channel the FileObject belongs to.
   * @param fo FileObject to save
   */
  public static void saveTempList(final String tmpfile, final String channel, final FileObject fo) throws IOException {
    // Create temp channel file.
    final File f = new File(tmpfile);
    boolean writeHeader = true;
    // If file already exists, we don't need a header, assuming that we just want to append the record.
    if (f.exists()) writeHeader = false;
   
    final FileWriter output = new FileWriter(f, true);
    if (writeHeader) { // Write header if required
      FileObjectWriter.writeFileHeader(output, channel);
    }
    FileObjectWriter.writeFileObject(output, fo);
    output.close();
  }
}
